# Generated by roxygen2: do not edit by hand

S3method(adjust,CovHR)
S3method(adjust,CoxScore)
S3method(adjust,GLMModel)
S3method(adjust,LogRank)
S3method(adjust,MHModel)
S3method(adjust,SLModel)
S3method(descript,GLMModelResult)
S3method(jacobian,CUSTOM)
S3method(jacobian,DIFF)
S3method(jacobian,LOGODDSRATIO)
S3method(jacobian,LOGRATIO)
S3method(jacobian,ODDSRATIO)
S3method(jacobian,RATIO)
S3method(print,CalibrationResult)
S3method(print,ContrastResult)
S3method(print,GLMModelResult)
S3method(print,LinModelResult)
S3method(print,MHResult)
S3method(print,TTEResult)
S3method(validate,RoboDataGLM)
S3method(validate,RoboDataLinear)
S3method(validate,RoboDataMH)
S3method(validate,RoboDataSL)
S3method(validate,RoboDataTTE)
export("%>%")
export(car_pb)
export(car_ps)
export(car_sr)
export(data_gen)
export(data_gen2)
export(robincar_SL)
export(robincar_SL_median)
export(robincar_calibrate)
export(robincar_contrast)
export(robincar_covhr)
export(robincar_coxscore)
export(robincar_glm)
export(robincar_linear)
export(robincar_logrank)
export(robincar_mh)
export(robincar_tte)
import(AIPW)
import(SuperLearner)
import(broom)
import(data.table)
import(fastDummies)
import(tidyverse)
importFrom(MASS,glm.nb)
importFrom(Rdpack,reprompt)
importFrom(SuperLearner,listWrappers)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,data.table)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(dplyr,arrange)
importFrom(dplyr,cur_group_id)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_all)
importFrom(dplyr,group_modify)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_all)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(emulator,quad.tform)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
importFrom(stats,cov)
importFrom(stats,model.matrix)
importFrom(stats,rbinom)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(stats,weighted.mean)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(utils,combn)
